/* Standard in kernel modules */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/fs.h>
#include <linux/modversions.h>
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/timer.h>
#include <linux/utime.h>
#include <mvGppRegs.h>

#include <mvGpp.h>
#include <mvCommon.h>
#include <mvTypes.h>

#define RESET_STR "RESET NOW!"
#define REBOOT_STR "REBOOT NOW!"
#define RESET_DEFAULT_FILE_NAME "resetDefault"
#define WAN_LINK_DEFAULT_FILE_NAME "wanLink"
#if FOR_DUALBAND
#define MODE_FILE_NAME "mode"
#define AOSS_FILE_NAME "aoss"
#endif
#define INTER_REGS_BASE							0xF1000000 /* mvSysHwConfig.h */
#if 0
#define GPIO_DATA_OUT_REG_OFFSET					0x10100
#define GPIO_DATE_OUT_ENABLE_CTRL_REG_OFFSET	0x10104
#define GPIO_DATA_IN_REG_OFFSET					0x10110
#define GPIO_DATA_IN_POLARITY_REG_OFFSET			0x1010c
#endif

#define GPIO_GROUP_0 0
#if 0
#define NO_BIT      0x00000000
#define BIT0        0x00000001
#define BIT1        0x00000002
#define BIT2        0x00000004
#define BIT3        0x00000008
#define BIT4        0x00000010
#define BIT5        0x00000020
#define BIT6        0x00000040
#define BIT7        0x00000080
#define BIT8        0x00000100
#define BIT9        0x00000200
#define BIT10       0x00000400
#define BIT11       0x00000800
#define BIT12       0x00001000
#define BIT13       0x00002000
#define BIT14       0x00004000
#define BIT15       0x00008000
#define BIT16       0x00010000
#define BIT17       0x00020000
#define BIT18       0x00040000
#define BIT19       0x00080000
#define BIT20       0x00100000
#define BIT21       0x00200000
#define BIT22       0x00400000
#define BIT23       0x00800000
#define BIT24       0x01000000
#define BIT25       0x02000000
#define BIT26       0x04000000
#define BIT27       0x08000000
#define BIT28       0x10000000
#define BIT29       0x20000000
#define BIT30       0x40000000
#define BIT31       0x80000000
#endif
#if 0
#define DEBUGP(format, args...) printk(KERN_EMERG"%s:%s: " format, __FILE__, __FUNCTION__, ## args)
#else
#define DEBUGP(x, args...)
#endif

#define MODULE_NAME "GPIO Control"

#define GPIO_TOTAL 19
#define GPIO_TEXT_LEN 10
#define RESET_TIME	5
#define RESET_GPIO_PIN MV_GPP1
struct timer_list Reset;
#if FOR_DUALBAND
/* Buttons */
#define MODE_GPIO_PIN MV_GPP20
#define AOSS_GPIO_PIN MV_GPP0
/* LEDs */
#define BRIDGE_LED_PIN MV_GPP21
#define AOSS_LED_PIN MV_GPP5
#define DIAG_LED_PIN MV_GPP7
struct timer_list Mode_detect;
struct timer_list Aoss_detect;
#endif
int	ResetCount = 0;
int	change = 0;

struct GPIO_data_d {
	char GPIO_NAME[GPIO_TEXT_LEN];
	unsigned long GPIO_PIN_NUM;
}GPIO_data[GPIO_TOTAL];

unsigned char GPIO_LIST_NAME[GPIO_TOTAL][GPIO_TEXT_LEN] =
{
	"GPIO0" ,
	"GPIO1" ,
	"GPIO2" ,
	"GPIO3" ,
	"GPIO4" ,
	"GPIO5" ,
	"GPIO6" ,
	"GPIO7" ,
	"GPIO17" ,
	"GPIO18" ,
	"GPIO19" ,
	"GPIO20" ,
	"GPIO21" ,
	"GPIO22" ,
	"GPIO23" ,
	"GPIO27" ,
	"GPIO28" ,
	"GPIO29" ,
	"GPIO30" ,
};

//char GPIO_LIST_PIN[GPIO_TOTAL] =
unsigned long GPIO_LIST_PIN[GPIO_TOTAL] =
{
	MV_GPP0 ,
	MV_GPP1 ,
	MV_GPP2 ,
	MV_GPP3 ,
	MV_GPP4 ,
	MV_GPP5 ,
	MV_GPP6 ,
	MV_GPP7 ,
	MV_GPP17 ,
	MV_GPP18 ,
	MV_GPP19 ,
	MV_GPP20 ,
	MV_GPP21 ,
	MV_GPP22 ,
	MV_GPP23 ,
	MV_GPP27 ,
	MV_GPP28 ,
	MV_GPP29 ,
	MV_GPP30 ,
};

char btn_action[24] = "";

#if FOR_DUALBAND
char mode_str[24] = "";
char aoss_str[24] = "";
#endif

#if 0
    #define MV_16BIT_LE(X)  (X) 
    #define MV_32BIT_LE(X)  (X)
    #define MV_64BIT_LE(X)  (X)
    #define MV_16BIT_BE(X)  MV_BYTE_SWAP_16BIT(X)
    #define MV_32BIT_BE(X)  MV_BYTE_SWAP_32BIT(X)
    #define MV_64BIT_BE(X)  MV_BYTE_SWAP_64BIT(X)
#endif



/* No Fast Swap implementation (in assembler) for ARM */
#define MV_32BIT_LE_FAST(val)            MV_32BIT_LE(val)
#define MV_16BIT_LE_FAST(val)            MV_16BIT_LE(val)
#define MV_32BIT_BE_FAST(val)            MV_32BIT_BE(val)
#define MV_16BIT_BE_FAST(val)            MV_16BIT_BE(val)


#if 0
/* 32bit read in little endian mode */
static __inline MV_U32 MV_MEMIO_LE32_READ(MV_U32 addr)
{
	MV_U32 data;

	data= (MV_U32)MV_MEMIO32_READ(addr);

	return (MV_U32)MV_32BIT_LE_FAST(data);
}
#endif

/* 32bit write in little endian mode */
#define MV_MEMIO_LE32_WRITE(addr, data) \
        MV_MEMIO32_WRITE(addr, MV_32BIT_LE_FAST(data))



#define MV_REG_READ(offset)         	\
        (MV_MEMIO_LE32_READ(INTER_REGS_BASE | (offset)))

#define MV_REG_WRITE(offset, val)    \
        MV_MEMIO_LE32_WRITE((INTER_REGS_BASE | (offset)), (val))






MV_VOID gppRegSet (MV_U32 group, MV_U32 regOffs,MV_U32 mask,MV_U32 value);


MV_STATUS mvGppTypeSet(MV_U32 group, MV_U32 mask, MV_U32 value)
{
#if 0
	if (group >= MV_GPP_MAX_GROUP)
	{
		mvOsPrintf("mvGppTypeSet: ERR. invalid group number \n");
		return MV_BAD_PARAM;
	}
#endif
	gppRegSet(group, GPP_DATA_OUT_EN_REG(group), mask, value);

	return MV_OK;

}

MV_VOID gppRegSet (MV_U32 group, MV_U32 regOffs,MV_U32 mask,MV_U32 value)
{
	MV_U32 gppData;

	gppData = MV_REG_READ(regOffs);

	gppData &= ~mask;

	gppData |= (value & mask);

	MV_REG_WRITE(regOffs, gppData);
}



/*******************************************************************************
* mvGppOutEnablle - Enable a GPP (OUT) pin
*
* DESCRIPTION:
*
* INPUT:
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the type
*               of corresponding GPP will be set. Other GPPs are ignored.
*       value - 32bit value that describes GPP type per pin.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Set GPP8 to input and GPP15 to output.
*       mvGppTypeSet(0, (GPP8 | GPP15), 
*                    ((MV_GPP_IN & GPP8) | (MV_GPP_OUT & GPP15)) );
*
* RETURN:
*       None.
*
*******************************************************************************/
MV_STATUS mvGppOutEnablle(MV_U32 group, MV_U32 mask, MV_U32 value)
{
#if 0
	if (group >= MV_GPP_MAX_GROUP)
	{
		mvOsPrintf("mvGppTypeSet: ERR. invalid group number \n");
		return MV_BAD_PARAM;
	}
#endif
	gppRegSet(group, GPP_DATA_OUT_EN_REG(group), mask, value);

	return MV_OK;

}


/*******************************************************************************
* mvGppValueGet - Get a GPP Pin list value.
*
* DESCRIPTION:
*       This function get GPP value.
*
* INPUT:
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the 
*               returned value is valid for it.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Get GPP8 and GPP15 value.
*       mvGppValueGet(0, (GPP8 | GPP15));
*
* RETURN:
*       32bit value that describes GPP activity mode per pin.
*
*******************************************************************************/
MV_U32 mvGppValueGet(MV_U32 group, MV_U32 mask)
{
	MV_U32 gppData;

	gppData = MV_REG_READ(GPP_DATA_IN_REG(group));

	gppData &= mask;

	return gppData;

}


MV_STATUS mvGppValueSet (MV_U32 group, MV_U32 mask, MV_U32 value)
{
	MV_U32 outEnable;
	MV_U32 i;


	/* verify that the gpp pin is configured as output 		*/
	/* Note that in the register out enabled -> bit = '0'. 	*/
	outEnable = ~MV_REG_READ(GPP_DATA_OUT_EN_REG(group));

	for (i = 0 ; i < 32 ;i++)
	{
		if (((mask & (1 << i)) & (outEnable & (1 << i))) != (mask & (1 << i)))
		{
			DEBUGP(KERN_EMERG"mvGppValueSet: Err. An attempt to set output value to GPP %d in input mode.(M)\n", i);
			return MV_ERROR;
		}
	}

	gppRegSet(group, GPP_DATA_OUT_REG(group), mask, value);

	return MV_OK;

}

MV_STATUS mvGppPolaritySet(MV_U32 group, MV_U32 mask, MV_U32 value)
{
	if (group >= MV_GPP_MAX_GROUP)
	{
		DEBUGP(mvOsPrintf("mvGppTypeSet: ERR. invalid group number \n"));
		return MV_BAD_PARAM;
	}

	gppRegSet(group, GPP_DATA_IN_POL_REG(group), mask, value);

	return MV_OK;

}



#define ETH_REG_BASE            0x72000
#define ETH_PORT_STATUS_REG(port)           (ETH_REG_BASE + 0x444 + ((port)<<10))
#define ETH_LINK_UP_BIT                     1
#define ETH_LINK_UP_MASK                    (1<<ETH_LINK_UP_BIT)


/* Make POWER LED Amber */
#define PWR_LED_AMBER() {mvGppOutEnablle(GPIO_GROUP_0,MV_GPP0,MV_GPP0&MV_GPP_OUT);\
					mvGppValueSet(GPIO_GROUP_0,MV_GPP0,MV_GPP0&1);}
/* Make POWER LED Green */
#define PWR_LED_GREEN() {mvGppOutEnablle(GPIO_GROUP_0,MV_GPP0,MV_GPP0&MV_GPP_OUT);\
					mvGppValueSet(GPIO_GROUP_0,MV_GPP0,MV_GPP0&0);}
/* Make POWER LED Blink */
#define PWR_LED_BLINK() {mvGppOutEnablle(GPIO_GROUP_0,MV_GPP2,MV_GPP2&MV_GPP_OUT);\
					mvGppValueSet(GPIO_GROUP_0,MV_GPP2,MV_GPP2&0);}
/* Make POWER LED Solid */
#define PWR_LED_SOLID() {mvGppOutEnablle(GPIO_GROUP_0,MV_GPP2,MV_GPP2&MV_GPP_IN);\
					mvGppValueSet(GPIO_GROUP_0,MV_GPP2,MV_GPP2&0);}

#if FOR_DUALBAND
/* Make  DIAG LED ON */
#define DIAG_LED_ON() {mvGppOutEnablle(GPIO_GROUP_0,DIAG_LED_PIN,DIAG_LED_PIN&MV_GPP_OUT);\
					mvGppPolaritySet(GPIO_GROUP_0, DIAG_LED_PIN, 0);\
					mvGppValueSet(GPIO_GROUP_0,DIAG_LED_PIN,DIAG_LED_PIN&0xffffffff);}
/* Make DIAG LED OFF */
#define DIAG_LED_OFF() {mvGppOutEnablle(GPIO_GROUP_0,DIAG_LED_PIN,DIAG_LED_PIN&MV_GPP_OUT);\
					mvGppPolaritySet(GPIO_GROUP_0, DIAG_LED_PIN, 0);\
					mvGppValueSet(GPIO_GROUP_0,DIAG_LED_PIN,DIAG_LED_PIN&0);}
/* Make BRIDGE LED ON */
#define BRIDGE_LED_ON() {mvGppOutEnablle(GPIO_GROUP_0,BRIDGE_LED_PIN,BRIDGE_LED_PIN&MV_GPP_OUT);\
					mvGppValueSet(GPIO_GROUP_0,BRIDGE_LED_PIN,BRIDGE_LED_PIN&0xffffffff);}
/* Make BRIDGE LED OFF */
#define BRIDGE_LED_OFF() {mvGppOutEnablle(GPIO_GROUP_0,BRIDGE_LED_PIN,BRIDGE_LED_PIN&MV_GPP_OUT);\
					mvGppValueSet(GPIO_GROUP_0,BRIDGE_LED_PIN,BRIDGE_LED_PIN&0);}
/* Make AOSS LED ON */
#define AOSS_LED_ON() {mvGppOutEnablle(GPIO_GROUP_0,AOSS_LED_PIN,AOSS_LED_PIN&MV_GPP_OUT);\
					mvGppValueSet(GPIO_GROUP_0,AOSS_LED_PIN,AOSS_LED_PIN&0xffffffff);}
/* Make AOSS LED OFF */
#define AOSS_LED_OFF() {mvGppOutEnablle(GPIO_GROUP_0,AOSS_LED_PIN,AOSS_LED_PIN&MV_GPP_OUT);\
					mvGppValueSet(GPIO_GROUP_0,AOSS_LED_PIN,AOSS_LED_PIN&0);}

#endif

static int reset_btn_read_proc(char *buf, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	len = sprintf(buf, btn_action);

	return len;
}

void ResetScan(void)
{
#if FOR_DUALBAND
	unsigned int ret = 0;

	ret =mvGppValueGet(GPIO_GROUP_0,RESET_GPIO_PIN);
	//DEBUGP(KERN_EMERG "reset_btn : reset_btn ret is %d\n", ret);

	if(!ret) //Button is pressed.
	{
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d...press\n", ResetCount);
		DIAG_LED_ON();
		sprintf(btn_action, "on (0x%X)\n", MV_REG_READ(MPP_CONTROL_REG0) );
	}
	else //Button is released.
	{
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d..release\n", ResetCount);
		DIAG_LED_OFF();		
		sprintf(btn_action, "off (0x%X)\n", MV_REG_READ(MPP_CONTROL_REG0) );
	}

#else
	unsigned int ret = 0;

	ret =mvGppValueGet(GPIO_GROUP_0,RESET_GPIO_PIN);
	//DEBUGP(KERN_EMERG "reset_btn : reset_btn ret is %d\n", ret);

	if(!ret) //Button is pressed.
	{
		ResetCount++;
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d...press\n", ResetCount);

		PWR_LED_BLINK();
		if(ResetCount>=5)
		{ 
			DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d > 5...press\n", ResetCount);
			sprintf(btn_action, RESET_STR);
			PWR_LED_AMBER();
			PWR_LED_SOLID();
		}
	}
	else //Button is released.
	{
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d..release\n", ResetCount);
		if(ResetCount > 0 && ResetCount < RESET_TIME)
		{
			DEBUGP(KERN_EMERG "reset_btn : %d,reset_btn reboot\n", ResetCount);
			sprintf(btn_action, REBOOT_STR);
			PWR_LED_AMBER();
			//reboot();
		}
		ResetCount = 0;
	}
#endif	
	return;
}

void ResetCheck(void)
{
	ResetScan();
	Reset.expires = (jiffies + HZ);
	add_timer(&Reset);
	return;
}

#if FOR_DUALBAND
static int mode_btn_read_proc(char *buf, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	len = sprintf(buf, mode_str);

	return len;
}
static int aoss_btn_read_proc(char *buf, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	len = sprintf(buf, aoss_str);

	return len;
}
void ModeScan(void)
{
	unsigned int ret = 0;

	ret =mvGppValueGet(GPIO_GROUP_0,MODE_GPIO_PIN);
	//DEBUGP(KERN_EMERG "reset_btn : reset_btn ret is %d\n", ret);

	if(!ret) //Button is pressed.
	{
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d...press\n", ResetCount);
		BRIDGE_LED_ON();
		sprintf(mode_str, "on\n");
		sprintf(mode_str, "on (0x%X)\n", MV_REG_READ(DEV_MULTI_CONTROL) );
	}
	else //Button is released.
	{
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d..release\n", ResetCount);
		BRIDGE_LED_OFF();		
		sprintf(mode_str, "off\n");
		sprintf(mode_str, "off (0x%X)\n", MV_REG_READ(DEV_MULTI_CONTROL) );
	}
	return;
}

void Mode_detect_fn(void)
{
	ModeScan();
	Mode_detect.expires = (jiffies + HZ);
	add_timer(&Mode_detect);
	return;
}

void AossScan(void)
{
	unsigned int ret = 0;

	ret =mvGppValueGet(GPIO_GROUP_0,AOSS_GPIO_PIN);
	//DEBUGP(KERN_EMERG "reset_btn : reset_btn ret is %d\n", ret);

	if(!ret) //Button is pressed.
	{
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d...press\n", ResetCount);
		AOSS_LED_ON();
		sprintf(aoss_str, "on\n");
		sprintf(aoss_str, "on (0x%X)\n", MV_REG_READ(MPP_CONTROL_REG0) );
	}
	else //Button is released.
	{
		//DEBUGP(KERN_EMERG "reset_btn : reset_btn is %d..release\n", ResetCount);
		AOSS_LED_OFF();		
		sprintf(aoss_str, "off\n");
		sprintf(aoss_str, "off (0x%X)\n", MV_REG_READ(MPP_CONTROL_REG0) );
	}
	return;
}

void Aoss_detect_fn(void)
{
	AossScan();
	Aoss_detect.expires = (jiffies + HZ);
	add_timer(&Aoss_detect);
	return;
}

#endif

struct proc_dir_entry *create_proc_read_write_entry(const char *name,
	mode_t mode, struct proc_dir_entry *base, 
	read_proc_t *read_proc, write_proc_t *write_proc, void * data)
{
	struct proc_dir_entry *res=create_proc_entry(name,mode,base);
	if (res) {
		res->read_proc=read_proc;
		res->write_proc=write_proc;
		res->data=data;
	}
	return res;
}

int GPIO_write_proc(struct file *file, const char __user *buffer, unsigned long count, void *data)
{
	char buf[10] = "";
	int len = 0;
	struct GPIO_data_d *gpio = (struct GPIO_data_d *)data;
	
	DEBUGP("GPIO_read_proc : got [%s], pin num is %d\n", gpio->GPIO_NAME, gpio->GPIO_PIN_NUM);
	
	DEBUGP("GPIO_write_proc : get %lu byte.\n", count);
	if(count >= 10)
	{
		len = 9;
	}
	else
	{
		len = count;
	}
	
	DEBUGP("GPIO_write_proc : after check ==> get %d byte.\n", len);
	
	if(copy_from_user(buf, buffer, len))
	{
		DEBUGP("GPIO_write_proc : fail.\n");		
		return -EFAULT;
	}
	
	DEBUGP("GPIO_write_proc : get data...[%s].\n", buf);
	
	if(buf[0] == '0')
	{
		mvGppOutEnablle(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&MV_GPP_OUT);
		mvGppValueSet(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&0);
//		gpio_line_config(gpio->GPIO_PIN_NUM,IXP425_GPIO_OUT);	
//		gpio_line_set(gpio->GPIO_PIN_NUM, IXP4XX_GPIO_LOW);
		DEBUGP("GPIO_write_proc : set to OUT LOW.\n");
	}
	else if(buf[0] == '1')
	{	
		mvGppOutEnablle(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&MV_GPP_OUT);
		mvGppValueSet(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&1);
//		gpio_line_config(gpio->GPIO_PIN_NUM,IXP425_GPIO_OUT);			
//		gpio_line_set(gpio->GPIO_PIN_NUM, IXP4XX_GPIO_HIGH);
		DEBUGP("GPIO_write_proc : set to OUT HIGH.\n");
	}
	else if(buf[0] == '2')
	{
		mvGppOutEnablle(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&MV_GPP_IN);
		mvGppValueSet(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&0);
//		gpio_line_config(gpio->GPIO_PIN_NUM,IXP425_GPIO_IN);	
//		gpio_line_set(gpio->GPIO_PIN_NUM, IXP4XX_GPIO_LOW);
		DEBUGP("GPIO_write_proc : set to IN LOW.\n");
	}
	else if(buf[0] == '3')
	{
		mvGppOutEnablle(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&MV_GPP_IN);
		mvGppValueSet(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&1);
//		gpio_line_config(gpio->GPIO_PIN_NUM,IXP425_GPIO_IN);			
//		gpio_line_set(gpio->GPIO_PIN_NUM, IXP4XX_GPIO_HIGH);
		DEBUGP("GPIO_write_proc : set to IN HIGH.\n");
	}
	else if(buf[0] == '4')
	{
		mvGppOutEnablle(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&MV_GPP_OUT);
		mvGppValueSet(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&0xffffffff);
		DEBUGP("GPIO_write_proc : set to OUT HIGH.\n");
	}
	else if(buf[0] == '5')
	{
		mvGppOutEnablle(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,gpio->GPIO_PIN_NUM&MV_GPP_OUT);
		mvGppValueSet(GPIO_GROUP_0,gpio->GPIO_PIN_NUM,0xffffffff);
		DEBUGP("GPIO_write_proc : set to OUT HIGH.\n");
	}
	else
	{
		DEBUGP("GPIO_write_proc : Unknown...do nothing.\n");
	}
	
	return len;
}  
			   
          
int GPIO_read_proc(char *buf, char **start, off_t off, int count, int *eof, void *data)
{
	int ret = 0;
	int len = 0;
	struct GPIO_data_d *gpio = (struct GPIO_data_d *)data;
	
	DEBUGP("GPIO_read_proc : got [%s], pin num is %d\n", gpio->GPIO_NAME, gpio->GPIO_PIN_NUM);
//	gpio_line_get(gpio->GPIO_PIN_NUM ,&ret);
	ret = mvGppValueGet(GPIO_GROUP_0,gpio->GPIO_PIN_NUM);
	len = sprintf(buf, "%d\n", ret);
	
	return len;
}

static int __init my_init(void) 
{
	int i;

	DEBUGP("init\n");

	/* Configure the ResetToDefault pin to input */
	mvGppOutEnablle(GPIO_GROUP_0,RESET_GPIO_PIN,RESET_GPIO_PIN&MV_GPP_IN);

	init_timer(&Reset);
	Reset.expires = (jiffies + HZ);
	Reset.function = ResetCheck;
	add_timer(&Reset);

	create_proc_read_entry(RESET_DEFAULT_FILE_NAME,0,NULL,reset_btn_read_proc,NULL);
#if FOR_DUALBAND
	/* Make MPP20 to MPP23 to GPIOs. */
	MV_REG_WRITE(DEV_MULTI_CONTROL, MV_REG_READ(DEV_MULTI_CONTROL) | MODE_GPIO_PIN | BRIDGE_LED_PIN | MV_GPP22 | MV_GPP23);

	mvGppOutEnablle(GPIO_GROUP_0,DIAG_LED_PIN,0);
	mvGppPolaritySet(GPIO_GROUP_0, DIAG_LED_PIN, 0);

	MV_U32 temp;
	temp = MV_REG_READ( MPP_CONTROL_REG0 );
	//temp &= ~(0xFF000000); //clear MPP 6 & 7
	temp &= ~(0xF0000000); //clear 7
	MV_REG_WRITE(MPP_CONTROL_REG0,temp );
	mvGppOutEnablle(0, MV_GPP7, 0);
	/* set GPP polarity */
	mvGppPolaritySet(0, MV_GPP7, 0);

	/* Configure the MODE pin to input */
	mvGppOutEnablle(GPIO_GROUP_0,MODE_GPIO_PIN,MODE_GPIO_PIN&MV_GPP_IN);
	mvGppValueSet(GPIO_GROUP_0,MODE_GPIO_PIN,MODE_GPIO_PIN&0);
	init_timer(&Mode_detect);
	Mode_detect.expires = (jiffies + HZ);
	Mode_detect.function = Mode_detect_fn;
	add_timer(&Mode_detect);

	create_proc_read_entry(MODE_FILE_NAME,0,NULL,mode_btn_read_proc,NULL);

	/* Configure the AOSS pin to input */
	mvGppOutEnablle(GPIO_GROUP_0,AOSS_GPIO_PIN,AOSS_GPIO_PIN&MV_GPP_IN);
	mvGppValueSet(GPIO_GROUP_0,AOSS_GPIO_PIN,AOSS_GPIO_PIN&0);
	
	init_timer(&Aoss_detect);
	Aoss_detect.expires = (jiffies + HZ);
	Aoss_detect.function = Aoss_detect_fn;
	add_timer(&Aoss_detect);

	create_proc_read_entry(AOSS_FILE_NAME,0,NULL,aoss_btn_read_proc,NULL);
#endif

	for(i = 0;i < GPIO_TOTAL; i++)
	{
		strcpy(GPIO_data[i].GPIO_NAME, GPIO_LIST_NAME[i]);
		GPIO_data[i].GPIO_PIN_NUM = GPIO_LIST_PIN[i];
		if(create_proc_read_write_entry(GPIO_data[i].GPIO_NAME,0,NULL,GPIO_read_proc,GPIO_write_proc,(void *)&GPIO_data[i]) == NULL)
		{
			DEBUGP("%s : fail\n", GPIO_data[i].GPIO_NAME);
		}
		else
		{
			DEBUGP("%s : success\n", GPIO_data[i].GPIO_NAME);			
		}
	}
	
	return 0;		
}

static void __exit my_fini(void) 
{
	int i;
	
	DEBUGP("gpio.c : cleanup_module\n");
	
	for(i = 0;i < GPIO_TOTAL; i++)
	{
		strcpy(GPIO_data[i].GPIO_NAME, GPIO_LIST_NAME[i]);
		remove_proc_entry(GPIO_data[i].GPIO_NAME, NULL);
	}
	del_timer(&Reset);
	remove_proc_entry(RESET_DEFAULT_FILE_NAME,NULL);
#if FOR_DUALBAND
	del_timer(&Mode_detect);
	del_timer(&Aoss_detect);
	remove_proc_entry(MODE_FILE_NAME,NULL);
	remove_proc_entry(AOSS_FILE_NAME,NULL);
#endif
	//remove_proc_entry("wanLink",NULL);	
}
module_init(my_init);
module_exit(my_fini);
MODULE_AUTHOR("Joshua");
MODULE_LICENSE("Gemtek");
EXPORT_NO_SYMBOLS ;

